#include-once
#include <String.au3>
#include <Crypt.au3>

#cs ----------------------------------------------------------------------------

	AutoIt Version: 3.3.6.1
	Author:         Matwachich

	Script Function:
		UDF Simplifiant la gestion de paramtres de configuration

	Description:
		Cet UDF gre pour vous les IniRead/IniDelete ou RegRead/RegDelete.

		- Il faut d'abord excuter _AutoCfg_Init(), en spcifiant le stockage utilis
		(Registre/Ini), et le chemin: soit fichier ini ou clef de registre, et d'autres
		options.

		- Puis on ajoute des entres ( faire  chaque lancement du programme) avec
		_AutoCfg_AddEntry(), en spcifiant la valeur par dfaut

		- Enfin, quand on a fini d'ajout des entres, on excute _AutoCfg_ReadAll()
		Pour placer chaque couple entre/valeur dans des variables, et pourvoir en suite
		les rcuprer dans le programme.
		Ici, pour chaque entre, si il xiste une valeur lui correspondant dans le support
		de stockage, elle lui sera associ, sinon, c'est sa valeur par dfaut
		(spcifi dans _AutoCfg_AddEntry()) qui lui sera associ

		- Dans le script, on utilise CFG() pour rcuprer une valeur.

		- _AutoCfg_SetEntry() sert  crire une valeur pour une entre, ou  supprimer
		la valeur associ  une entre, pour la remtre  sa valeur par dfaut
		
	Nouvelle version: 13/04/2011
		- Refonte totale du code source
		- Il n'est plus possible de spcifier des entres crypt, on peut soit crypter
			toutes les entres (en donnant une valeur autre que "" au parametre
			$sPwd de _AutoCfg_Init()), ou n'en crypter aucune.
		- Les ConsoleWrite on t retirs
		- CFG() sans paramtre retourn un array 2D avec toutes les entres de configuration
			$array[0][0] = Nombre d'entres
			$array[$i][0] = Nom de l'entre
			$array[$i][1] = Valeur actuelle de l'entre
			$array[$i][2] = Valeur par dfaut de l'entre

#ce ----------------------------------------------------------------------------

Global Const $ACFG_REG = 0x01
Global Const $ACFG_INI = 0x02
Global Const $ACFG_NOENTRY = 0x03

Global $__sMainPwd = -1, $__sIniSection = -1, $__sPath = -1, $__iType = -1, $__iCry = -1, $__AutoCFG_FileSize = -1
Global $__ACFG_MainArray[1][2] ; name, default
	$__ACFG_MainArray[0][0] = 0

; # FUNCTION # ==============================================================================================
; Name...........:	_AutoCfg_Init
; Description....:	Initialisation
; Parameters.....:	$iType = Type de stockage:
;						$ACFG_REG - Base de registre
;						$ACFG_INI - Fichier .ini
;					$sPath = Chemin du stockage (Clef de registre ou fichier .ini) selon $iType
;					$sPwd = Si:
;						"" - Pas de cryptage des entres
;						Autre - Cryptage des entres activ, et la clef de cryptage sera la chaine de $sPwd
;					$sSectionName = Nom de la section, si fichier .ini
; Return values..:	Succes - 1
;					Echec - 0 Et @error:
;									-1 -> Mauvais $iType
;									 1 -> Dj initialis
; Author.........:	Matwachich
; Remarks........:	
; ===========================================================================================================
Func _AutoCfg_Init($iType, $sPath, $sPwd = "", $sSectionName = "", $iUpdTime = 0)
	If $__sPath <> -1 Then Return SetError(1, 0, 0)
	; ---
	Switch $iType
		Case $ACFG_INI, $ACFG_REG
			$__iType = $iType
			$__sPath = $sPath
			$__sIniSection = $sSectionName
		Case Else
			Return SetError(-1, 0, 0)
	EndSwitch
	; ---
	Switch $sPwd
		Case ""
			$__iCry = 0
			$__sMainPwd = ""
		Case Else
			$__iCry = 1
			$__sMainPwd = $sPwd
	EndSwitch
	; ---
	Return SetError(0, 0, 1)
EndFunc

; # FUNCTION # ==============================================================================================
; Name...........:	_AutoCfg_AddEntry
; Description....:	Ajoute une entre, avec sa valeur par defaut
; Parameters.....:	$sName = Nom de l'entre
;					$sDefault = Valeur par dfaut
; Return values..:	Succes - 1
;					Echec - 0 Et @error = 1 -> Pas initialis
; Author.........:	Matwachich
; Remarks........:	
; ===========================================================================================================
Func _AutoCfg_AddEntry($sName, $sDefault)
	If $__sPath = -1 Then Return SetError(1, 0, 0)
	; ---
	Local $uB = $__ACFG_MainArray[0][0]
	ReDim $__ACFG_MainArray[$uB + 2][2]
	$__ACFG_MainArray[$uB + 1][0] = $sName
	$__ACFG_MainArray[$uB + 1][1] = $sDefault
	$__ACFG_MainArray[0][0] += 1
	Return SetError(0, 0, 1)
EndFunc

; # FUNCTION # ==============================================================================================
; Name...........:	_AutoCfg_ReadAll
; Description....:	Initialise toutes les entres soit avec leurs valeurs correspondantes
;						dans le support de stockage, ou avec la valeur par dfaut
; Parameters.....:	Aucun
; Return values..:	Succes - 1
;					Echec - 0 Et @error = 1 -> Pas initialis
; Author.........:	
; Remarks........:	
; ===========================================================================================================
Func _AutoCfg_ReadAll()
	If $__sPath = -1 Then Return SetError(1, 0, 0)
	; ---
	Local $read
	For $i = 1 To $__ACFG_MainArray[0][0]
		Switch $__iType
			Case $ACFG_INI
				$read = IniRead($__sPath, $__sIniSection, $__ACFG_MainArray[$i][0], $ACFG_NOENTRY)
				If $read <> $ACFG_NOENTRY Then
					If $__iCry Then
						$read = _StringEncrypt(0, $read, $__sMainPwd)
					EndIf
				EndIf
				__ACFG_Assign($__ACFG_MainArray[$i][0], $read, $__ACFG_MainArray[$i][1])
			Case $ACFG_REG
				$read = RegRead($__sPath, $__ACFG_MainArray[$i][0])
				If @error Then 
					$read = $ACFG_NOENTRY
				Else
					If $__iCry Then
						$read = _StringEncrypt(0, $read, $__sMainPwd)
					EndIf
				EndIf
				__ACFG_Assign($__ACFG_MainArray[$i][0], $read, $__ACFG_MainArray[$i][1])
		EndSwitch
	Next
	; ---
	Return SetError(0, 0, 1)
EndFunc

; # FUNCTION # ==============================================================================================
; Name...........:	_AutoCfg_SetEntry
; Description....:	Donne une valeur  une entre, ou la remet  sa valeur par dfaut
; Parameters.....:	$sName = Nom de l'entre
;					$sValue = Valeur  assigner.
;						Si laiss par dfaut -> remet l'entre  sa valeur par dfaut
; Return values..:	Succes - 1
;					Echec - 0 Et @error = 1 -> Pas initialis
; Author.........:	Matwachich
; Remarks........:	
; ===========================================================================================================
Func _AutoCfg_SetEntry($sName, $sValue = $ACFG_NOENTRY)
	If $__sPath = -1 Then Return SetError(1, 0, 0)
	; ---
	Local $toAssign, $toWrite
	If $sValue <> $ACFG_NOENTRY Then
		If $__iCry Then
			$toWrite = _StringEncrypt(1, $sValue, $__sMainPwd)
		Else
			$toWrite = $sValue
		EndIf
		$toAssign = $sValue
		; ---
		__ACFG_Assign($sName, $toAssign, 1) ; Juste pour viter la recherche automatique de la valeur par dfaut
		__ACFG_Write($sName, $toWrite)
	Else
		__ACFG_Assign($sName, $ACFG_NOENTRY)
		__ACFG_Write($sName)
	EndIf
	; ---
	Return 1
EndFunc

; # FUNCTION # ==============================================================================================
; Name...........:	CFG
; Description....:	Rcupre la valeur contenue dans une entr
; Parameters.....:	$sName = Nom de l'entre
; Return values..:	Succes - Valeur de l'entr
;					Echec - 0 et @error:
;									1 -> Pas initialis
;									2 -> Le nom d'entr spcifier n'existe pas
; Author.........:	Matwachich
; Remarks........:	
; ===========================================================================================================
Func CFG($sName = Default)
	If $__sPath = -1 Then Return SetError(1, 0, 0)
	; ---
	If $sName = Default And IsKeyword($sName) Then Return __ACFG_GetAll()
	; ---
	Local $read = Eval("__ACFG_VARPREFIX_" & $sName)
	If @error Then Return SetError(2, 0, 0)
	Return SetError(0, 0, $read)
EndFunc

; ==============================================================
; ### Internals
; ==============================================================

Func __ACFG_GetAll()
	Local $array[$__ACFG_MainArray[0][0] + 1][3]
		$array[0][0] = 1
	For $i = 1 To $__ACFG_MainArray[0][0]
		$array[$i][0] = $__ACFG_MainArray[$i][0]
		$array[$i][1] = CFG($__ACFG_MainArray[$i][0])
		$array[$i][2] = $__ACFG_MainArray[$i][1]
		; ---
		$array[0][0] += 1
	Next
	Return $array
EndFunc

Func __ACFG_Assign($sName, $sRead, $sDefault = Default)
	Local $toAssign
	If $sDefault = Default And IsKeyword($sDefault) Then $sDefault = __ACFG_ValueGetDefault($sName)
	; ---
	If $sRead = $ACFG_NOENTRY Then
		$toAssign = $sDefault
	Else
		$toAssign = $sRead
	EndIf
	; ---
	Assign("__ACFG_VARPREFIX_" & $sName, $toAssign, 2)
EndFunc

Func __ACFG_Write($sName, $sValue = $ACFG_NOENTRY)
	If $sValue <> $ACFG_NOENTRY Then
		Switch $__iType
			Case $ACFG_INI
				IniWrite($__sPath, $__sIniSection, $sName, $sValue)
			Case $ACFG_REG
				RegWrite($__sPath, $sName, "REG_SZ", $sValue)
		EndSwitch
	Else
		Switch $__iType
			Case $ACFG_INI
				IniDelete($__sPath, $__sIniSection, $sName)
			Case $ACFG_REG
				RegDelete($__sPath, $sName)
		EndSwitch
	EndIF
EndFunc

Func __ACFG_ValueGetDefault($sName)
	For $i = 1 To $__ACFG_MainArray[0][0]
		If $__ACFG_MainArray[$i][0] = $sName Then Return $__ACFG_MainArray[$i][1]
	Next
	Return $ACFG_NOENTRY
EndFunc